/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qstring.h>
#include <qstringlist.h>
#include <qregexp.h>

#include <bibtexentry.h>
#include <bibtexfile.h>
#include <bibtexxsltransform.h>
#include <bibtexentryfield.h>

#define max(a,b) ((a)>(b)?(a):(b))

namespace BibTeX
{

    BibTeXEntry::BibTeXEntry( )
            : BibTeXElement(), m_entryType( etUnknown ), m_entryTypeString( QString::null ), m_id( QString::null )
    {
        // nothing
    }

    BibTeXEntry::BibTeXEntry( const EntryType entryType, const QString &id )
            : BibTeXElement( ), m_entryType( entryType ), m_id( id )
    {
        m_entryTypeString = entryTypeToString( entryType );
    }

    BibTeXEntry::BibTeXEntry( const QString& entryTypeString, const QString& id ) : BibTeXElement( ), m_entryTypeString( entryTypeString ), m_id( id )
    {
        m_entryType = entryTypeFromString( entryTypeString );
    }

    BibTeXEntry::BibTeXEntry( BibTeXEntry *other )
    {
        m_entryType = other->m_entryType;
        m_entryTypeString = other->m_entryTypeString;
        m_id = other->m_id;
        for ( EntryFields::iterator it = other->m_fields.begin(); it != other->m_fields.end(); it++ )
        {
            m_fields.append( new BibTeXEntryField( *it ) );
        }
    }

    BibTeXEntry::~BibTeXEntry()
    {
        for ( EntryFields::iterator it = m_fields.begin(); it != m_fields.end(); it++ )
        {
            delete( *it );
        }
    }

    void BibTeXEntry::setEntryType( const EntryType entryType )
    {
        m_entryType = entryType;
        m_entryTypeString = entryTypeToString( entryType );
    }

    void BibTeXEntry::setEntryTypeString( const QString& entryTypeString )
    {
        m_entryTypeString = entryTypeString;
        m_entryType = entryTypeFromString( entryTypeString );
    }

    BibTeXEntry::EntryType BibTeXEntry::entryType()
    {
        return m_entryType;
    }

    QString BibTeXEntry::entryTypeString()
    {
        return m_entryTypeString;
    }

    void BibTeXEntry::setId( const QString& id )
    {
        m_id = id;
    }

    QString BibTeXEntry::id()
    {
        return m_id;
    }

    bool BibTeXEntry::containsPattern( const QString & pattern, bool caseSensitive )
    {
        bool result = m_id.find( pattern, 0, caseSensitive ) > 0;

        for ( EntryFields::iterator it = m_fields.begin(); !result && it != m_fields.end(); it++ )
            for ( BibTeX::BibTeXElement::FieldItems::iterator fit = ( *it ) ->begin(); !result && fit != ( *it ) ->end(); fit++ )
                result = ( *fit ).text.find( pattern, 0, caseSensitive ) > -1;

        return result;
    }

    bool BibTeXEntry::addField( BibTeXEntryField * field )
    {
        m_fields.append( field );
        return TRUE;
    }

    BibTeXEntryField* BibTeXEntry::getField( const BibTeXEntryField::FieldType fieldType )
    {
        BibTeXEntryField * result = NULL;

        for ( EntryFields::iterator it = m_fields.begin(); ( it != m_fields.end() ) && ( result == NULL ); it++ )
            if ( ( *it ) ->fieldType() == fieldType ) result = *it;

        return result;
    }

    BibTeXEntryField* BibTeXEntry::getField( const QString & fieldName )
    {
        BibTeXEntryField * result = NULL;

        for ( EntryFields::iterator it = m_fields.begin(); ( it != m_fields.end() ) && ( result == NULL ); it++ )
            if ( ( *it ) ->fieldTypeName().lower() == fieldName.lower() )
                result = *it;

        return result;
    }

    bool BibTeXEntry::deleteField( const QString & fieldName )
    {
        for ( EntryFields::iterator it = m_fields.begin(); it != m_fields.end(); it++ )
            if ( ( *it ) ->fieldTypeName().lower() == fieldName.lower() )
            {
                delete( *it );
                m_fields.remove( it );
                return TRUE;
            }

        return FALSE;
    }

    bool BibTeXEntry::deleteField( const BibTeXEntryField::FieldType fieldType )
    {
        for ( EntryFields::iterator it = m_fields.begin(); it != m_fields.end(); it++ )
            if ( ( *it ) ->fieldType() == fieldType )
            {
                delete( *it );
                m_fields.remove( it );
                return TRUE;
            }

        return FALSE;
    }

    BibTeXEntry::EntryFields::iterator BibTeXEntry::begin()
    {
        return m_fields.begin();
    }

    BibTeXEntry::EntryFields::iterator BibTeXEntry::end()
    {
        return m_fields.end();
    }

    int BibTeXEntry::getFieldCount()
    {
        return m_fields.count();
    }

    void BibTeXEntry::clearFields()
    {
        for ( EntryFields::iterator it = m_fields.begin(); it != m_fields.end(); it++ )
            delete( *it );
        m_fields.clear();
    }

    QString BibTeXEntry::entryTypeToString( const EntryType entryType )
    {
        switch ( entryType )
        {
        case etArticle:
            return QString( "Article" );
        case etBook:
            return QString( "Book" );
        case etBooklet:
            return QString( "Booklet" );
        case etCollection:
            return QString( "Collection" );
        case etElectronic:
            return QString( "Electronic (IEEE)" );
        case etInBook:
            return QString( "InBook" );
        case etInCollection:
            return QString( "InCollection" );
        case etInProceedings:
            return QString( "InProceedings" );
        case etManual:
            return QString( "Manual" );
        case etMastersThesis:
            return QString( "MastersThesis" );
        case etMisc:
            return QString( "Misc" );
        case etPhDThesis:
            return QString( "PhDThesis" );
        case etProceedings:
            return QString( "Proceedings" );
        case etTechReport:
            return QString( "TechReport" );
        case etUnpublished:
            return QString( "Unpublished" );
        default:
            return QString( "Unknown" );
        }
    }

    BibTeXEntry::EntryType BibTeXEntry::entryTypeFromString( const QString & entryTypeString )
    {
        QString entryTypeStringLower = entryTypeString.lower();
        if ( entryTypeStringLower.compare( "article" ) == 0 )
            return etArticle;
        else if ( entryTypeStringLower.compare( "book" ) == 0 )
            return etBook;
        else if ( entryTypeStringLower.compare( "booklet" ) == 0 )
            return etBooklet;
        else if ( entryTypeStringLower.compare( "collection" ) == 0 )
            return etCollection;
        else if ( ( entryTypeStringLower.compare( "electronic" ) == 0 ) || ( entryTypeStringLower.compare( "online" ) == 0 ) || ( entryTypeStringLower.compare( "internet" ) == 0 ) || ( entryTypeStringLower.compare( "webpage" ) == 0 ) )
            return etElectronic;
        else if ( entryTypeStringLower.compare( "inbook" ) == 0 )
            return etInBook;
        else if ( entryTypeStringLower.compare( "incollection" ) == 0 )
            return etInCollection;
        else if ( ( entryTypeStringLower.compare( "inproceedings" ) == 0 ) || ( entryTypeStringLower.compare( "conference" ) == 0 ) )
            return etInProceedings;
        else if ( entryTypeStringLower.compare( "manual" ) == 0 )
            return etManual;
        else if ( entryTypeStringLower.compare( "mastersthesis" ) == 0 )
            return etMastersThesis;
        else if ( entryTypeStringLower.compare( "misc" ) == 0 )
            return etMisc;
        else if ( entryTypeStringLower.compare( "phdthesis" ) == 0 )
            return etPhDThesis;
        else if ( entryTypeStringLower.compare( "proceedings" ) == 0 )
            return etProceedings;
        else if ( entryTypeStringLower.compare( "techreport" ) == 0 )
            return etTechReport;
        else if ( entryTypeStringLower.compare( "unpublished" ) == 0 )
            return etUnpublished;
        else
            return etUnknown;
    }

    BibTeXEntryField::FieldRequireStatus BibTeXEntry::getRequireStatus( BibTeXEntry::EntryType entryType, BibTeXEntryField::FieldType fieldType )
    {
        switch ( entryType )
        {
        case etArticle:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftJournal:
            case BibTeXEntryField::ftYear:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftVolume:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftNumber:
            case BibTeXEntryField::ftPages:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftISSN:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etBook:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftEditor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftPublisher:
            case BibTeXEntryField::ftYear:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftVolume:
            case BibTeXEntryField::ftSeries:
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftEdition:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftISBN:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etBooklet:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftTitle:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftHowPublished:
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftYear:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftISBN:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etElectronic:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftHowPublished:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftYear:
            case BibTeXEntryField::ftKey:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etInBook:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftEditor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftPages:
            case BibTeXEntryField::ftPublisher:
            case BibTeXEntryField::ftYear:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftVolume:
            case BibTeXEntryField::ftSeries:
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftEdition:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftCrossRef:
            case BibTeXEntryField::ftChapter:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftISBN:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etInCollection:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftBookTitle:
            case BibTeXEntryField::ftPublisher:
            case BibTeXEntryField::ftYear:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftEditor:
            case BibTeXEntryField::ftPages:
            case BibTeXEntryField::ftOrganization:
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftType:
            case BibTeXEntryField::ftISBN:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etInProceedings:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftYear:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftPages:
            case BibTeXEntryField::ftBookTitle:
            case BibTeXEntryField::ftEditor:
            case BibTeXEntryField::ftVolume:
            case BibTeXEntryField::ftNumber:
            case BibTeXEntryField::ftSeries:
            case BibTeXEntryField::ftType:
            case BibTeXEntryField::ftChapter:
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftEdition:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftCrossRef:
            case BibTeXEntryField::ftPublisher:
            case BibTeXEntryField::ftISBN:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etManual:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftTitle:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftOrganization:
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftEdition:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftYear:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftISBN:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etMastersThesis:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftSchool:
            case BibTeXEntryField::ftYear:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etMisc:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftHowPublished:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftYear:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etPhDThesis:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftSchool:
            case BibTeXEntryField::ftYear:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftISBN:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etCollection:
        case etProceedings:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftYear:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftEditor:
            case BibTeXEntryField::ftPublisher:
            case BibTeXEntryField::ftOrganization:
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftSeries:
            case BibTeXEntryField::ftBookTitle:
            case BibTeXEntryField::ftISBN:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etTechReport:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftInstitution:
            case BibTeXEntryField::ftYear:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftType:
            case BibTeXEntryField::ftNumber:
            case BibTeXEntryField::ftAddress:
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftNote:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        case etUnpublished:
            switch ( fieldType )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftTitle:
            case BibTeXEntryField::ftNote:
                return BibTeXEntryField::frsRequired;
            case BibTeXEntryField::ftMonth:
            case BibTeXEntryField::ftYear:
            case BibTeXEntryField::ftKey:
            case BibTeXEntryField::ftURL:
                return BibTeXEntryField::frsOptional;
            default: return BibTeXEntryField::frsIgnored;
            }
        default:
            return BibTeXEntryField::frsOptional;
        }
    }

}
