/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <bibtexfile.h>
#include <bibtexelement.h>
#include <bibtexentry.h>
#include <bibtexstring.h>
#include <bibtexcomment.h>
#include <encoderlatex.h>

#include "bibtexfileexporterbibtex.h"

namespace BibTeX
{

    BibTeXFileExporterBibTeX::BibTeXFileExporterBibTeX() : BibTeXFileExporter()
    {
        cancelFlag = FALSE;
        m_stringOpenDelimiter = '"';
        m_stringCloseDelimiter = '"';
    }


    BibTeXFileExporterBibTeX::~BibTeXFileExporterBibTeX()
    {
        // nothing
    }


    bool BibTeXFileExporterBibTeX::save( QIODevice* iodevice, BibTeXFile* bibtexfile )
    {
        bool result = TRUE;
        QTextStream stream( iodevice );

        writePreamble( stream, bibtexfile->preamble() );

        // first, write strings at the beginning
        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXString *string = dynamic_cast<BibTeXString*>( *it );
            if ( string != NULL )
                result &= writeString( stream, string );
        }

        // second, write cross-referencing elements
        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( *it );
            if ( ( entry != NULL ) && ( entry->getField( BibTeXEntryField::ftCrossRef ) != NULL ) )
                result &= writeEntry( stream, entry );
        }

        // third, write remaining elements
        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( *it );
            if ( entry != NULL )
            {
                if ( entry->getField( BibTeXEntryField::ftCrossRef ) == NULL )
                    result &= writeEntry( stream, entry );
            }
            else
            {
                BibTeXComment *comment = dynamic_cast<BibTeXComment*>( *it );
                if ( comment != NULL )
                    result &= writeComment( stream, comment );
            }
        }

        return result && !cancelFlag;
    }

    bool BibTeXFileExporterBibTeX::save( QIODevice* iodevice, BibTeXElement* element )
    {
        bool result = FALSE;
        QTextStream stream( iodevice );
        BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( element );
        if ( entry != NULL )
            result |= writeEntry( stream, entry );
        else
        {
            BibTeXString * string = dynamic_cast<BibTeXString*>( element );
            if ( string != NULL )
                result |= writeString( stream, string );
            else
            {
                BibTeXComment * comment = dynamic_cast<BibTeXComment*>( element );
                if ( comment != NULL )
                    result |= writeComment( stream, comment );
            }
        }

        return result;
    }

    void BibTeXFileExporterBibTeX::cancel()
    {
        cancelFlag = TRUE;
    }

    bool BibTeXFileExporterBibTeX::writeEntry( QTextStream&stream, BibTeXEntry* entry )
    {
        stream << "@" << entry->entryTypeString() << "{ " << entry->id();

        for ( BibTeXEntry::EntryFields::iterator it = entry->begin(); it != entry->end(); it++ )
        {
            BibTeXEntryField *field = *it;
            stream << ',' << endl << '\t' << field->fieldTypeName() << " = " << itemToString( field->begin(), field->end(), field->fieldType() );
        }
        stream << endl << "}" << endl << endl;
        return TRUE;
    }

    bool BibTeXFileExporterBibTeX::writeString( QTextStream&stream, BibTeXString *string )
    {
        stream << "@string{ " << string->key() << " = " << itemToString( string->begin(), string->end() ) << " }" << endl << endl;

        return TRUE;
    }

    bool BibTeXFileExporterBibTeX::writeComment( QTextStream&stream, BibTeXComment *comment )
    {
        if ( comment->usePercent() )
        {
            QStringList commentLines = QStringList::split( '\n', EncoderLaTeX::currentEncoderLaTeX() ->encode( comment->text() ) );
            for ( QStringList::Iterator it = commentLines.begin(); it != commentLines.end(); it++ )
            {
                //                 qDebug( "writeComment %s", ( *it ).latin1() );
                stream << '%' << *it << endl;
            }
            stream << endl;
        }
        else
        {
            stream << "@comment{" << EncoderLaTeX::currentEncoderLaTeX() ->encode( comment->text() ) << "}" << endl << endl;
        }
        return TRUE;
    }

    bool BibTeXFileExporterBibTeX::writePreamble( QTextStream &stream, const QString& preamble )
    {
        if ( !preamble.isEmpty() )
            stream << "@preamble{" << EncoderLaTeX::currentEncoderLaTeX() ->encode( preamble ) << "}" << endl << endl;
        return TRUE;
    }

    void BibTeXFileExporterBibTeX::setStringDelimiter( const QChar& stringOpenDelimiter, const QChar& stringCloseDelimiter )
    {
        m_stringOpenDelimiter = stringOpenDelimiter;
        m_stringCloseDelimiter = stringCloseDelimiter;
    }

    QString BibTeXFileExporterBibTeX::itemToString( const BibTeXElement::FieldItems::iterator& begin, const BibTeXElement::FieldItems::iterator& end, const BibTeXEntryField::FieldType fieldType )
    {
        QString result;
        bool inString = FALSE;
        bool isFirst = TRUE;
        EncoderLaTeX *encoder = EncoderLaTeX::currentEncoderLaTeX();

        for ( BibTeXElement::FieldItems::const_iterator it = begin; it != end; it++ )
        {
            BibTeXElement::FieldItem fieldItem = *it;
            if ( !inString && !fieldItem.isStringKey )
            {
                if ( !isFirst )
                    result.append( " # " );
                result.append( m_stringOpenDelimiter );
                inString = TRUE;
            }
            else if ( inString && fieldItem.isStringKey )
            {
                result.append( m_stringCloseDelimiter );
                if ( !isFirst )
                    result.append( " # " );
                inString = FALSE;
            }

            if ( fieldItem.isStringKey )
                result.append( fieldItem.text );
            else
            {
                result.append( encoder->encodeSpecialized( fieldItem.text, fieldType ) );
            }

            isFirst = FALSE;
        }

        if ( inString )
            result.append( m_stringCloseDelimiter );
        return result;
    }

}
