/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <string>

#include <qstring.h>
#include <qsplitter.h>
#include <qtimer.h>
#include <qptrlist.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qbuffer.h>
#include <qtoolbutton.h>
#include <qtabwidget.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qprogressdialog.h>
#include <qclipboard.h>
#include <kinstance.h>
#include <kaction.h>
#include <kactionclasses.h>
#include <kstdaction.h>
#include <klistview.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <ktempfile.h>
#include <kconfigdialog.h>
#include <kglobalsettings.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <kiconloader.h>
#include <kapplication.h>
#include <kxmlguifactory.h>
#include <kio/netaccess.h>
#include <kpopupmenu.h>

#include <bibtexcomment.h>
#include <bibtexxsltransform.h>
#include <bibtexstring.h>
#include <bibtexentry.h>
#include <bibtexelement.h>
#include <kbibtexlistviewitem.h>
#include <kbibtexsettingsdlg.h>
#include <kbibtexlistview.h>
#include <bibtexfileexporter.h>
#include <bibtexfileexporterbibtex.h>
#include <bibtexfileexporterxml.h>
#include <bibtexfileexporterxslt.h>
#include <bibtexfileimporter.h>
#include <bibtexfileimporterbibtex.h>
#include <settings.h>
#include <encoderlatex.h>

#include "kbibtex_part.h"

Settings kbibtexsettings;

KBibTeXPart::KBibTeXPart( QWidget *parentWidget, const char *widgetName,
                          QObject *parent, const char *name )
        : KParts::ReadWritePart( parent, name ), m_settingsDlg( NULL ), m_bibTeXXSLTHTML( NULL )
{
    // we need an instance
    setInstance( KBibTeXPartFactory::instance() );

    // set our XML-UI resource file
    setXMLFile( "kbibtex_part.rc" );

    // setup GUI elements
    setupGUI( parentWidget, widgetName );

    // create our actions
    setupActions();

    // we are read-write by default
    setReadWrite( TRUE );

    // we are not modified since we haven't done anything yet
    setModified( FALSE );

    // read configuration
    readSettings();

    updateMenu();

    QTimer::singleShot( 1000, this, SLOT( slotDeferredInitialization() ) );
}

KBibTeXPart::~KBibTeXPart()
{
    // save configuration
    writeSettings();

    if ( m_settingsDlg != NULL )
        delete m_settingsDlg;

    if ( m_bibTeXXSLTHTML != NULL )
        delete m_bibTeXXSLTHTML;

    BibTeX::EncoderLaTeX::deleteCurrentEncoderLaTeX();
}

void KBibTeXPart::setReadWrite( bool rw )
{
    ReadWritePart::setReadWrite( rw );
}

void KBibTeXPart::setModified( bool modified )
{
    // get a handle on our Save action and make sure it is valid
    KAction * save = actionCollection() ->action( KStdAction::stdName( KStdAction::Save ) );

    if ( save )
    {
        // if so, we either enable or disable it based on the current
        // state
        save->setEnabled( modified );

        // in any event, we want our parent to do it's thing
        KParts::ReadWritePart::setModified( modified );
    }
}

bool KBibTeXPart::openFile()
{
    bool result = m_url.isValid() && m_bibtexlist->open( m_url, FALSE );
    if ( result )
        setModified( FALSE );
    else
        KMessageBox::error( widget(), QString( i18n( "Opening file \"%1\" failed." ) ).arg( m_url.prettyURL() ), i18n( "Error opening file" ) );

    return result;
}

bool KBibTeXPart::saveFile()
{
    qDebug( "save url: %s", m_url.prettyURL().latin1() );
    return FALSE;
}

bool KBibTeXPart::fileOpen()
{
    m_bibtexlist->setEnabled( FALSE );
    KURL url = KFileDialog::getOpenURL( QString::null, "*.bib|" + i18n( "BibTeX (*.bib)" ), widget() );

    bool result = url.isValid();
    if ( result )
    {
        if ( ( result = m_bibtexlist->open( url, FALSE ) ) )
        {
            m_url = url;
            m_recent->addURL( url );
            emit setWindowCaption( url.prettyURL() );
            setModified( FALSE );
        }
        else
            KMessageBox::error( widget(), QString( i18n( "Opening file \"%1\" failed." ) ).arg( url.prettyURL() ), i18n( "Error opening file" ) );
    }
    m_bibtexlist->setEnabled( TRUE );

    return result;
}

bool KBibTeXPart::fileInsert()
{
    m_bibtexlist->setEnabled( FALSE );
    KURL url = KFileDialog::getOpenURL( QString::null, "*.bib|" + i18n( "BibTeX (*.bib)" ), widget() );

    bool result = url.isValid();
    if ( result )
    {
        if ( ( result = m_bibtexlist->open( url, TRUE ) ) )
        {
            m_recent->addURL( url );
            setModified( TRUE );
        }
        else
            KMessageBox::error( widget(), QString( i18n( "Inserting file \"%1\" failed." ) ).arg( url.prettyURL() ), i18n( "Error inserting file" ) );
    }

    m_bibtexlist->setEnabled( TRUE );

    return result;
}

bool KBibTeXPart::fileImport()
{
    if ( kbibtexsettings.bibconvAvailable )
    {
        m_bibtexlist->setEnabled( FALSE );
        KURL url = KFileDialog::getOpenURL( QString::null, "*.enl|" + i18n( "EndNote Library (*.enl)" ) + "\n*.pdt|" + i18n( "Procite (*.pdt)" ), widget() );

        bool result = url.isValid();
        if ( result )
        {
            if ( ( result = m_bibtexlist->open( url, FALSE ) ) )
            {
                m_recent->addURL( url );
                setModified( TRUE );
            }
            else
                KMessageBox::error( widget(), QString( i18n( "Importing file \"%1\" failed." ) ).arg( url.prettyURL() ), i18n( "Error importing file" ) );
        }

        m_bibtexlist->setEnabled( TRUE );

        return result;
    }
    else
        return FALSE;
}

bool KBibTeXPart::fileSave()
{
    bool result = m_url.isValid();

    if ( result )
    {
        m_bibtexlist->setEnabled( FALSE );
        if ( m_bibtexlist->save( m_url, Settings::formatBibTeX, FALSE ) )
        {
            m_recent->addURL( m_url );
            setModified( FALSE );
            result = TRUE;
        }
        else
            KMessageBox::error( widget(), QString( i18n( "Saving file \"%1\" failed." ) ).arg( m_url.prettyURL() ), i18n( "Error saving file" ) );

        m_bibtexlist->setEnabled( TRUE );
    }
    else
        result = fileSaveAs();

    return result;
}

bool KBibTeXPart::fileSaveAs()
{
    m_bibtexlist->setEnabled( FALSE );
    KURL url = KFileDialog::getSaveURL( QString::null, "*.bib|" + i18n( "BibTeX (*.bib)" ), widget() );

    bool result = url.isValid() && ( !KIO::NetAccess::exists( url, TRUE, widget() ) || KMessageBox::warningContinueCancel( widget(), QString( i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" ) ).arg( url.prettyURL() ), i18n( "Overwrite File?" ), i18n( "&Overwrite" ) ) == KMessageBox::Continue );

    if ( result )
    {
        if ( ( result = m_bibtexlist->save( url, Settings::formatBibTeX, FALSE ) ) )
        {
            m_recent->addURL( url );
            m_url = url;
            emit setWindowCaption( url.prettyURL() );
            setModified( FALSE );
        }
        else
            KMessageBox::error( widget(), QString( i18n( "Saving file \"%1\" failed." ) ).arg( url.prettyURL() ), i18n( "Error saving file" ) );
    }

    m_bibtexlist->setEnabled( TRUE );
    return result;
}

bool KBibTeXPart::fileSaveSelectedAs()
{
    m_bibtexlist->setEnabled( FALSE );
    KURL url = KFileDialog::getSaveURL( QString::null, "*.bib|" + i18n( "BibTeX (*.bib)" ), widget() );

    bool result = url.isValid() && ( !KIO::NetAccess::exists( url, TRUE, widget() ) || KMessageBox::warningContinueCancel( widget(), QString( i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" ) ).arg( url.prettyURL() ), i18n( "Overwrite File?" ), i18n( "&Overwrite" ) ) == KMessageBox::Continue );

    if ( result )
    {
        if ( ( result = m_bibtexlist->save( url, Settings::formatBibTeX, TRUE ) ) )
        {
            m_recent->addURL( url );
            result = TRUE;
        }
        else
            KMessageBox::error( widget(), QString( i18n( "Saving file \"%1\" failed." ) ).arg( url.prettyURL() ), i18n( "Error saving file" ) );
    }

    m_bibtexlist->setEnabled( TRUE );
    return result;
}

bool KBibTeXPart::fileExport()
{
    m_bibtexlist->setEnabled( FALSE );
    QString formatString = "*.pdf|" + i18n( "Portable Document File (*.pdf)" ) + "\n*.ps|" + i18n( "PostScript (*.ps)" ) + "\n*.xml|" + i18n( "eXtended Markup Language (*.xml)" ) + "\n*.html|" + i18n( "Hypertext Markup Langage (*.html)" );
    if ( kbibtexsettings.bibconvAvailable )
        formatString.append( "\n*.enl|" + i18n( "EndNote Library (*.enl)" ) + "\n*.pdt|" + i18n( "ProCite (*.pdt)" ) );
    KURL url = KFileDialog::getSaveURL( QString::null, formatString, widget() );

    bool result = url.isValid() && ( !KIO::NetAccess::exists( url, TRUE, widget() ) || KMessageBox::warningContinueCancel( widget(), QString( i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" ) ).arg( url.prettyURL() ), i18n( "Overwrite File?" ), i18n( "&Overwrite" ) ) == KMessageBox::Continue );

    if ( result )
    {
        Settings::FileFormat format = Settings::formatUndefined;
        QString fileName = url.fileName();

        if ( fileName.endsWith( ".pdf" ) )
            format = Settings::formatPDF;
        else if ( fileName.endsWith( ".ps" ) )
            format = Settings::formatPS;
        else if ( fileName.endsWith( ".xml" ) )
            format = Settings::formatXML;
        else if ( fileName.endsWith( ".html" ) )
            format = Settings::formatHTML;
        else if ( kbibtexsettings.bibconvAvailable && fileName.endsWith( ".enl" ) )
            format = Settings::formatEndNote;
        else if ( kbibtexsettings.bibconvAvailable && fileName.endsWith( ".pdt" ) )
            format = Settings::formatProCite;

        if ( format != Settings::formatUndefined )
            result = m_bibtexlist->save( url, format, FALSE );
        else
            result = FALSE;

        if ( !result )
            KMessageBox::error( widget(), QString( i18n( "Exporting file \"%1\" failed." ) ).arg( url.prettyURL() ), i18n( "Error exporting file" ) );
    }

    m_bibtexlist->setEnabled( TRUE );
    return result;
}

bool KBibTeXPart::fileExportSelected()
{
    m_bibtexlist->setEnabled( FALSE );
    QString formatString = "*.pdf|" + i18n( "Portable Document File (*.pdf)" ) + "\n*.ps|" + i18n( "PostScript (*.ps)" ) + "\n*.xml|" + i18n( "eXtended Markup Language (*.xml)" ) + "\n*.html|" + i18n( "Hypertext Markup Langage (*.html)" );
    if ( kbibtexsettings.bibconvAvailable )
        formatString.append( "\n*.enl|" + i18n( "EndNote Library (*.enl)" ) + "\n*.pdt|" + i18n( "ProCite (*.pdt)" ) );
    KURL url = KFileDialog::getSaveURL( QString::null, formatString, widget() );

    bool result = url.isValid() && ( !KIO::NetAccess::exists( url, TRUE, widget() ) || KMessageBox::warningContinueCancel( widget(), QString( i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" ) ).arg( url.prettyURL() ), i18n( "Overwrite File?" ), i18n( "&Overwrite" ) ) == KMessageBox::Continue );

    if ( result )
    {
        Settings::FileFormat format = Settings::formatUndefined;
        QString fileName = url.fileName();
        if ( fileName.endsWith( ".pdf" ) )
            format = Settings::formatPDF;
        else if ( fileName.endsWith( ".ps" ) )
            format = Settings::formatPS;
        else if ( fileName.endsWith( ".xml" ) )
            format = Settings::formatXML;
        else if ( fileName.endsWith( ".html" ) )
            format = Settings::formatHTML;
        else if ( kbibtexsettings.bibconvAvailable && fileName.endsWith( ".enl" ) )
            format = Settings::formatEndNote;
        else if ( kbibtexsettings.bibconvAvailable && fileName.endsWith( ".pdt" ) )
            format = Settings::formatProCite;

        if ( format != Settings::formatUndefined )
            result = m_bibtexlist->save( url, format, TRUE );
        else
            result = FALSE;

        if ( !result )
            KMessageBox::error( widget(), QString( i18n( "Exporting file \"%1\" failed." ) ).arg( url.prettyURL() ), i18n( "Error exporting file" ) );
    }

    m_bibtexlist->setEnabled( TRUE );
    return result;
}

void KBibTeXPart::setupGUI( QWidget *parentWidget, const char *name )
{
    m_centralWidget = new QTabWidget( parentWidget );

    // List view tab widget ===============================

    QWidget *verticalContainer = new QWidget( m_centralWidget );
    QBoxLayout *layout = new QVBoxLayout( verticalContainer, 0, 0 );
    m_centralWidget->addTab( verticalContainer, i18n( "List view" ) );

    m_searchBar = new KBibTeX::SearchBar( verticalContainer );
    layout->addWidget( m_searchBar );

    QSplitter * splitter = new QSplitter( Qt::Vertical, verticalContainer );
    layout->addWidget( splitter );

    m_bibtexlist = new KBibTeXListView( this, splitter, name );
    m_preview = new QLabel( splitter );
    m_preview->setAutoResize( FALSE );

    // Source view tab widget =============================

    m_sourceView = new QTextEdit( splitter );
    m_centralWidget->addTab( m_sourceView, i18n( "Source view" ) );
    m_sourceView->setFont( KGlobalSettings::fixedFont() );

    // notify the part that this is our internal widget
    setWidget( m_centralWidget );
}

void KBibTeXPart::setupActions()
{
    KStdAction::open( this, SLOT( fileOpen() ), actionCollection() );
    m_actionFileInsert = new KAction( i18n( "Insert..." ), 0, this, SLOT( fileInsert() ), actionCollection(), "file_insert" );
    if ( kbibtexsettings.bibconvAvailable )
        m_actionFileImport = new KAction( i18n( "Import..." ), 0, this, SLOT( fileImport() ), actionCollection(), "file_import" );
    KStdAction::save( this, SLOT( fileSave() ), actionCollection() );
    KStdAction::saveAs( this, SLOT( fileSaveAs() ), actionCollection() );
    m_actionFileSaveSelectedAs = new KAction( i18n( "Save Selected As..." ), 0, this, SLOT( fileSaveSelectedAs() ), actionCollection(), "file_save_selected_as" );
    m_actionFileExport = new KAction( i18n( "Export..." ), SmallIconSet("goto"), CTRL + Key_E, this, SLOT( fileExport() ), actionCollection(), "file_export" );
    m_actionFileExportSelected = new KAction( i18n( "Export selected..." ), 0, this, SLOT( fileExportSelected() ), actionCollection(), "file_export_selected" );

    m_actionEditCut = KStdAction::cut( this, SLOT( cutElement() ), actionCollection() );
    m_actionEditCopy = KStdAction::copy( this, SLOT( copyElement() ), actionCollection() );
    m_actionEditPaste = KStdAction::paste( this, SLOT( pasteElement() ), actionCollection() );

    m_menuSearchInternet = new KActionMenu( i18n( "Search Internet" ), actionCollection(), "search_internet" );
    connect( m_menuSearchInternet->popupMenu(), SIGNAL( activated( int ) ), this, SLOT( slotSearchInternet( int ) ) );

    m_actionPreferences = KStdAction::preferences( this, SLOT( slotPreferences() ), actionCollection() );

    new KAction( i18n( "Article" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_article" );
    new KAction( i18n( "Book" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_book" );
    new KAction( i18n( "Booklet" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_booklet" );
    new KAction( i18n( "Electronic (IEEE)" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_electronic" );
    new KAction( i18n( "InBook" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_inbook" );
    new KAction( i18n( "InCollection" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_incollection" );
    new KAction( i18n( "InProceedings" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_inproceedings" );
    new KAction( i18n( "Manual" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_manual" );
    new KAction( i18n( "MastersThesis" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_mastersthesis" );
    new KAction( i18n( "Misc" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_misc" );
    new KAction( i18n( "PhDThesis" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_phdthesis" );
    new KAction( i18n( "Proceedings" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_proceedings" );
    new KAction( i18n( "TechReport" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_techreport" );
    new KAction( i18n( "Unpublished" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_unpublished" );
    new KAction( i18n( "Comment" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_comment" );
    new KAction( i18n( "String" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_string" );

    m_actionEditElement = new KAction( i18n( "&Edit" ), 0, this, SLOT( editElement() ), actionCollection(), "element_edit" );
    m_actionDeleteElement = new KAction( i18n( "&Delete" ), 0, this, SLOT( deleteElement() ), actionCollection(), "element_delete" );

    m_recent = new KRecentFilesAction( i18n( "Open &Recent Files" ), 0,
                                       this, SLOT( openURL( const KURL & ) ),
                                       actionCollection(), "file_open_recent" );

    m_actionShowComments = new KToggleAction( i18n( "Show &Comments" ), 0, this, SLOT( slotToggleShowSpecialElements() ), actionCollection(), "view_showcomments" );
    m_actionShowStrings = new KToggleAction( i18n( "Show &Strings" ), 0, this, SLOT( slotToggleShowSpecialElements() ), actionCollection(), "view_showstrings" );

    connect( m_bibtexlist, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( previewElement( QListViewItem* ) ) );
    connect( m_bibtexlist, SIGNAL( selectionChanged( ) ), this, SLOT( previewElement( ) ) );
    connect( m_bibtexlist, SIGNAL( doubleClicked( QListViewItem*, const QPoint&, int ) ), this, SLOT( editElement( QListViewItem* ) ) );
    connect( m_bibtexlist, SIGNAL( selectionChanged() ), this, SLOT( slotSelectionChanged( ) ) );
    connect( m_bibtexlist, SIGNAL( contextMenu( KListView *, QListViewItem *, const QPoint & ) ), this, SLOT( showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint & ) ) );
    connect( m_searchBar, SIGNAL( doSearch( const QString& ) ), m_bibtexlist, SLOT( filterText( const QString& ) ) );
    connect( m_centralWidget, SIGNAL( currentChanged ( QWidget * ) ), this, SLOT( slotTabChanged( QWidget* ) ) );
    connect( m_sourceView, SIGNAL( textChanged() ), this, SLOT( slotSourceChanged() ) );

    actionCollection() ->setHighlightingEnabled( true );

    KActionMenu *acmElementNew = new KActionMenu( i18n( "&New" ), "filenew", actionCollection(), "element_new" );
    acmElementNew->setEnabled( FALSE );
}

bool KBibTeXPart::newElement()
{
    if ( isReadWrite() && sender() )
    {
        QString elementType;

        if ( strncmp( sender() ->name(), "element_new_entry_", 18 ) == 0 )
            elementType = QString( sender() ->name() + 18 );
        else if ( strcmp( sender() ->name(), "element_new_comment" ) == 0 )
            elementType = "comment";
        else if ( strcmp( sender() ->name(), "element_new_string" ) == 0 )
            elementType = "string";
        else
            return FALSE;

        bool result = m_bibtexlist->newElement( elementType );
        if ( result )
            setModified( TRUE );
        return result;
    }

    return FALSE;
}

bool KBibTeXPart::previewElement( QListViewItem * item )
{
    bool result = FALSE;

    if ( item == NULL )
        item = m_bibtexlist->selectedItem();
    if ( item == NULL )
        item = m_bibtexlist->currentItem();
    if ( item == NULL )
        return FALSE;

    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( item );
    if ( kblvi != NULL && m_bibtexlist->isEnabled() )
    {
        BibTeX::BibTeXElement * element = kblvi->element();
        BibTeX::BibTeXFileExporter * exporter = new BibTeX::BibTeXFileExporterXSLT( m_bibTeXXSLTHTML );

        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        result = exporter->save( &buffer, element );
        buffer.close();

        if ( result )
        {
            buffer.open( IO_ReadOnly );
            QString htmlText = QTextStream( &buffer ).read();
            buffer.close();
            m_preview->setText( htmlText.remove( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" ).append( "</qt>" ).prepend( "<qt>" ).replace( '{', "" ).replace( '}', "" ) );
            //             m_preview->setText( QString( dummy ).remove( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" ).append( "</qt>" ).prepend( "<qt>" ) );
        }

        delete exporter;
    }

    return result;
}

bool KBibTeXPart::editElement( QListViewItem * item )
{
    bool result = FALSE;

    if ( item == NULL )
        item = m_bibtexlist->selectedItem();
    if ( item == NULL )
        item = m_bibtexlist->currentItem();
    if ( item == NULL )
        return false;

    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( item );
    if ( kblvi != NULL )
        result = m_bibtexlist->editElement( kblvi, isReadWrite() );
    if ( result )
        setModified( TRUE );

    return result;
}

void KBibTeXPart::deleteElement()
{
    if ( isReadWrite() )
    {
        m_bibtexlist->deleteSelected();
        setModified( TRUE );
    }
}

void KBibTeXPart::readSettings()
{
    KConfig * config = KBibTeXPartFactory::instance() ->config();

    config->setGroup( "Session" );
    m_recent->loadEntries( config );

    kbibtexsettings.load( config );
    m_bibtexlist->setSorting( kbibtexsettings.mainListSortingColumn );
    for ( int c = 0; c < 5; c++ )
        m_bibtexlist->setColumnWidth( c, kbibtexsettings.mainListColumnWidth[ c ] );
    m_bibtexlist->enabledActions();

    if ( m_bibTeXXSLTHTML != NULL )
        delete m_bibTeXXSLTHTML;
    m_bibTeXXSLTHTML = new BibTeX::BibTeXXSLTransform( kbibtexsettings.xsltStylesheetHTML );

    m_actionShowComments->setChecked( kbibtexsettings.showComments );
    m_actionShowStrings->setChecked( kbibtexsettings.showStrings );
}

void KBibTeXPart::writeSettings()
{
    KConfig * config = KBibTeXPartFactory::instance() ->config();

    config->setGroup( "Session" );
    m_recent->saveEntries( config );

    kbibtexsettings.save( config );
}

void KBibTeXPart::cutElement()
{
    if ( isReadWrite() )
    {
        m_bibtexlist->cutSelected();
        setModified( TRUE );
    }
}

void KBibTeXPart::copyElement()
{
    m_bibtexlist->copySelected();
}

void KBibTeXPart::pasteElement()
{
    if ( isReadWrite() )
    {
        m_bibtexlist->paste();
        setModified( TRUE );
    }
}

void KBibTeXPart::showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint & p )
{
    if ( factory() )
    {
        updateMenu();
        KPopupMenu* popupMenu = static_cast<KPopupMenu*>( factory() ->container( "popup_bibtexlist", this ) );
        popupMenu ->popup( p );
    }
}

void KBibTeXPart::updateMenu()
{
    int numSelected = 0;
    QListViewItemIterator it( m_bibtexlist, QListViewItemIterator::Selected );
    while ( it.current() && numSelected < 3 )
    {
        numSelected++;
        it++;
    }

    m_actionFileSaveSelectedAs->setEnabled( numSelected > 0 );
    m_actionFileExportSelected->setEnabled( numSelected > 0 );
    //     m_actionElementsList->setEnabled( isReadWrite() );
    m_actionEditElement->setEnabled( numSelected == 1 );
    m_actionDeleteElement->setEnabled( isReadWrite() && numSelected > 0 );
    m_actionEditCut->setEnabled( isReadWrite() && numSelected > 0 );
    m_actionEditCopy->setEnabled( numSelected > 0 );
    m_menuSearchInternet->setEnabled( numSelected == 1 );

    QPopupMenu *popup = m_menuSearchInternet->popupMenu();
    popup->clear();
    int i = 0;
    for ( Settings::SearchURL * searchURL = kbibtexsettings.searchURLs.first(); searchURL; searchURL = kbibtexsettings.searchURLs.next() )
        popup->insertItem( searchURL->description, ++i );
}

void KBibTeXPart::slotSelectionChanged()
{
    updateMenu();
}

void KBibTeXPart::slotSearchInternet( int id )
{
    KBibTeXListViewItem * lvi = dynamic_cast<KBibTeXListViewItem*>( m_bibtexlist->selectedItem() );
    if ( lvi == NULL )
        lvi = dynamic_cast<KBibTeXListViewItem*>( m_bibtexlist->currentItem() );

    if ( lvi != NULL )
        searchInternet( lvi->element(), kbibtexsettings.searchURLs.at( id - 1 ) ->url );
}

void KBibTeXPart::searchInternet( BibTeX::BibTeXElement * element, const QString& searchURL )
{
    QString queryString = QString::null;

    BibTeX::BibTeXEntry* entry = dynamic_cast<BibTeX::BibTeXEntry*>( element );
    if ( entry != NULL )
    {
        BibTeX::BibTeXEntryField * field = entry->getField( BibTeX::BibTeXEntryField::ftTitle );
        if ( field && field->begin() != field->end() )
            queryString = ( *( field->begin() ) ).text;
    }
    else
    {
        BibTeX::BibTeXComment * comment = dynamic_cast<BibTeX::BibTeXComment*>( element );
        if ( comment != NULL )
            queryString = comment->text();
        else
        {
            BibTeX::BibTeXString * string = dynamic_cast<BibTeX::BibTeXString*>( element );
            if ( string != NULL && string->begin() != string->end() )
                queryString = ( *( string->begin() ) ).text;
            else
                qDebug( "Not yet supported" );
        }
    }

    if ( queryString != QString::null )
        kapp->invokeBrowser( QString( searchURL ).arg( queryString.replace( '{', "" ).replace( '}', "" ) ) );
}

void KBibTeXPart::slotTabChanged( QWidget *tab )
{
    m_bibtexlist->setEnabled( FALSE );
    m_sourceView->setEnabled( FALSE );
    if ( tab == m_sourceView )
    {
        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        bool result = m_bibtexlist->save( &buffer, Settings::formatBibTeX, i18n( "Generating source view" ), FALSE );
        buffer.close();

        if ( result )
        {
            buffer.open( IO_ReadOnly );
            QString text = QTextStream( &buffer ).read();
            buffer.close();
            m_sourceView->setText( text );
            m_sourceView->setModified( FALSE );
        }
        m_sourceView->setFocus();
    }
    else if ( isModified() )
    {
        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        QTextStream stream( &buffer );
        stream << m_sourceView->text();
        buffer.close();
        buffer.open( IO_ReadOnly );
        m_bibtexlist->open( &buffer, FALSE, i18n( "Parsing source view" ) );
        buffer.close();
        if ( m_sourceView->isModified() )
            setModified( TRUE );
        m_bibtexlist->setFocus();
    }
    m_bibtexlist->setEnabled( TRUE );
    m_sourceView->setEnabled( TRUE );
}

void KBibTeXPart::slotSourceChanged()
{
    if ( m_sourceView->isEnabled() )
        setModified( TRUE );
}

void KBibTeXPart::slotToggleShowSpecialElements()
{
    kbibtexsettings.showComments = m_actionShowComments->isChecked();
    kbibtexsettings.showStrings = m_actionShowStrings->isChecked();

    m_bibtexlist->updateView();
}

void KBibTeXPart::slotPreferences()
{
    if ( !m_settingsDlg )
        m_settingsDlg = new KBibTeXSettingsDlg( widget(), "settings_dlg" );

    m_settingsDlg->exec();


}

bool KBibTeXPart::queryClose()
{
    if ( !isModified() )
        return true;

    switch ( KMessageBox::warningYesNoCancel( widget(), i18n( "The current document has been modified.\nDo you want to save your changes or discard them?" ), i18n( "Close Document" ), KStdGuiItem::save(), KStdGuiItem::discard() ) )
    {
    case KMessageBox::Yes:
        return save();
    case KMessageBox::No:
        return true;
    default:
        return false;
    }
}

void KBibTeXPart::slotDeferredInitialization()
{
    m_searchBar->setAddElementMenu( static_cast<KPopupMenu*>( factory() -> container( "popup_newelements", this ) ) );
}

// It's usually safe to leave the factory code alone.. with the
// notable exception of the KAboutData data
#include <kaboutdata.h>
#include <klocale.h>

KInstance* KBibTeXPartFactory::s_instance = 0L;
KAboutData* KBibTeXPartFactory::s_about = 0L;

KBibTeXPartFactory::KBibTeXPartFactory()
        : KParts::Factory()
{
    // nothing
}

KBibTeXPartFactory::~KBibTeXPartFactory()
{
    delete s_instance;
    delete s_about;

    s_instance = 0L;
}

KParts::Part* KBibTeXPartFactory::createPartObject( QWidget *parentWidget, const char *widgetName,
        QObject *parent, const char *name,
        const char *classname, const QStringList & /* args */ )
{
    // Create an instance of our Part
    KBibTeXPart * obj = new KBibTeXPart( parentWidget, widgetName, parent, name );

    // See if we are to be read-write or not
    if ( QCString( classname ) == "KParts::ReadOnlyPart" )
        obj->setReadWrite( false );

    return obj;
}

KInstance* KBibTeXPartFactory::instance()
{
    if ( !s_instance )
    {
        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "0.1.3" );
        s_about->addAuthor( "Thomas Fischer", 0, "fischer@unix-ag.uni-kl.de" );
        s_instance = new KInstance( s_about );
    }
    return s_instance;
}

extern "C"
{
    void* init_libkbibtexpart()
    {
        return new KBibTeXPartFactory;
    }
};

#include "kbibtex_part.moc"

