/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qevent.h>
#include <qdragobject.h>
#include <qfile.h>
#include <qapplication.h>
#include <qprogressdialog.h>
#include <qbuffer.h>
#include <qclipboard.h>
#include <qheader.h>
#include <qtextstream.h>
#include <kapplication.h>
#include <kio/netaccess.h>
#include <ktempfile.h>
#include <klocale.h>
#include <kurl.h>
#include <kurldrag.h>

#include <kbibtex_part.h>
#include <kbibtexlistviewitem.h>
#include <bibtexfile.h>
#include <bibtexelement.h>
#include <bibtexentry.h>
#include <bibtexstring.h>
#include <bibtexcomment.h>
#include <bibtexfileimporter.h>
#include <bibtexfileimporterexternal.h>
#include <bibtexfileimporterbibtex.h>
#include <bibtexfileexporter.h>
#include <bibtexfileexporterbibtex.h>
#include <bibtexfileexporterexternal.h>
#include <bibtexfileexporterxml.h>
#include <bibtexfileexporterpdf.h>
#include <bibtexfileexporterps.h>
#include <bibtexfileexporterxslt.h>
#include <kbibtexentrywidget.h>
#include <kbibtexcommentwidget.h>
#include <kbibtexstringwidget.h>
#include "kbibtexlistview.h"

KBibTeXListView::KBibTeXListView( KBibTeXPart *kbibtexPart, QWidget *parent, const char *name ) : KListView( parent, name ), m_kbibtexPart( kbibtexPart ), m_progressDialog( NULL ), m_newElementCounter( 1 )
{
    for ( int i = 0; i < ColumnsCount; i++ )
    {
        addColumn( Columns[ i ] );
        setColumnWidthMode( i, QListView::Manual );
    }

    setAllColumnsShowFocus( true );
    setShowSortIndicator( true );
    setSelectionMode( QListView::Extended );
    header() ->setClickEnabled ( TRUE );

    setAcceptDrops( TRUE );
    //     clipper() ->setAcceptDrops( TRUE );
    setDragEnabled( TRUE );

    m_bibtexfile = new BibTeX::BibTeXFile();
}

KBibTeXListView::~KBibTeXListView()
{
    delete m_bibtexfile;
}

void KBibTeXListView::setItems()
{
    QApplication::setOverrideCursor( Qt::waitCursor );
    setUpdatesEnabled( FALSE );

    clearAll();
    for ( unsigned int i = 0; i < m_bibtexfile->count(); i++ )
    {
        BibTeX::BibTeXElement *element = m_bibtexfile->at( i );
        QListViewItem *item = new KBibTeXListViewItem( element, this );
        item->setVisible( m_filter.isEmpty() || element->containsPattern( m_filter ) );
    }
    sort();

    setUpdatesEnabled( TRUE );
    QApplication::restoreOverrideCursor();
    triggerUpdate();
}

void KBibTeXListView::updateVisiblity()
{
    QListViewItemIterator it( this );
    while ( it.current() )
    {
        KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );
        kblvi->setVisible( m_filter.isEmpty() || kblvi->element() ->containsPattern( m_filter ) );

        BibTeX::BibTeXElement *element = kblvi->element();
        BibTeX::BibTeXString *string = dynamic_cast<BibTeX::BibTeXString*>( element );
        if ( string != NULL && kbibtexsettings.showStrings == FALSE )
            kblvi->setVisible( FALSE );
        else
        {
            BibTeX::BibTeXComment *comment = dynamic_cast<BibTeX::BibTeXComment*>( element );
            if ( comment != NULL && kbibtexsettings.showComments == FALSE )
                kblvi->setVisible( FALSE );
        }

        it++;
    }
}

bool KBibTeXListView::open( const KURL &url, bool insertOnly )
{
    Settings::FileFormat format = Settings::formatUndefined;
    QString fileName = url.fileName();
    if ( fileName.endsWith( ".bib" ) )
        format = Settings::formatBibTeX;
    else if ( fileName.endsWith( ".enl" ) )
        format = Settings::formatEndNote;
    else if ( fileName.endsWith( ".pdt" ) )
        format = Settings::formatProCite;
    else
        return FALSE;

    BibTeX::BibTeXFileImporter *importer = NULL;
    switch ( format )
    {
    case Settings::formatBibTeX:
        importer = new BibTeX::BibTeXFileImporterBibTeX();
        break;
    case Settings::formatEndNote:
    case Settings::formatProCite:
        importer = new BibTeX::BibTeXFileImporterExternal( Settings::importerBibConv, format );
        break;
    default:
        return FALSE;
    }

    QString tmpFile;
    bool result = FALSE;
    if ( KIO::NetAccess::download( url, tmpFile, this ) )
    {
        QFile file( tmpFile );
        if ( file.open( IO_ReadOnly ) )
        {
            result = open( &file, insertOnly, QString( i18n( "<qt>Loading file <b>%1</b></qt>" ) ).arg( url.fileName() ), importer );
            file.close();
        }
        KIO::NetAccess::removeTempFile( tmpFile );
    }
    delete importer;
    return result;
}

bool KBibTeXListView::open( QIODevice *iodevice, bool insertOnly, const QString &label, BibTeX::BibTeXFileImporter *importer )
{
    bool usingDefaultImporter = importer == NULL;
    if ( usingDefaultImporter )
        importer = new BibTeX::BibTeXFileImporterBibTeX();

    startProgress( label, importer );
    BibTeX::BibTeXFile *newFile = importer->load( iodevice );
    endProgress( importer );

    if ( usingDefaultImporter )
        delete importer;

    if ( newFile )
    {
        if ( insertOnly )
        {
            appendBibTeXFile( newFile );
            delete newFile;
        }
        else
        {
            delete m_bibtexfile;
            m_bibtexfile = newFile;
        }
        setItems();
        return TRUE;
    }
    else
    {
        qDebug( "Could not load bibliography file from io device" );
        return FALSE;
    }
}

bool KBibTeXListView::save( const KURL & url, Settings::FileFormat format, bool onlySelected )
{
    KTempFile tempfile = KTempFile();
    tempfile.setAutoDelete( TRUE );
    bool result = save( tempfile.file(), format, QString( i18n( "<qt>Writing file <b>%1</b></qt>" ) ).arg( url.fileName() ), onlySelected );
    tempfile.close();

    if ( result )
        result = KIO::NetAccess::upload( tempfile.name(), url, this );

    return result;
}

bool KBibTeXListView::save( QIODevice *iodevice, Settings::FileFormat format, const QString &label, bool onlySelected )
{
    BibTeX::BibTeXXSLTransform * transform = NULL;

    BibTeX::BibTeXFileExporter * exporter = NULL;
    switch ( format )
    {
    case Settings::formatBibTeX:
        exporter = new BibTeX::BibTeXFileExporterBibTeX();
        break;
    case Settings::formatXML:
        exporter = new BibTeX::BibTeXFileExporterXML();
        break;
    case Settings::formatHTML:
        switch ( kbibtexsettings.usedExporterHTML )
        {
        case Settings::exporterNone:
        case Settings::exporterXSLT:
            {
                transform = new BibTeX::BibTeXXSLTransform( kbibtexsettings.xsltStylesheetHTML );
                if ( transform != NULL )
                    exporter = new BibTeX::BibTeXFileExporterXSLT( transform );
                else
                {
                    qDebug( "XSLT for HTML export is not available" );
                    return FALSE;
                }
            }
            break;
        default:
            exporter = new BibTeX::BibTeXFileExporterExternal( kbibtexsettings.usedExporterHTML, Settings::formatHTML );
        }
        break;
    case Settings::formatPDF:
        exporter = new BibTeX::BibTeXFileExporterPDF();
        break;
    case Settings::formatPS:
        exporter = new BibTeX::BibTeXFileExporterPS();
        break;
    case Settings::formatEndNote:
    case Settings::formatProCite:
        exporter = new BibTeX::BibTeXFileExporterExternal( kbibtexsettings.exporterBibConv, format );
        break;
    default:
        qDebug( "No supported export format selected" );
        return FALSE;
    }

    startProgress( label, exporter );
    bool result = FALSE;
    if ( onlySelected )
    {
        BibTeX::BibTeXFile * bibtexfile = selected();
        result = exporter->save( iodevice, bibtexfile );
        delete bibtexfile;
    }
    else
        result = exporter->save( iodevice, m_bibtexfile );
    endProgress( exporter );

    if ( transform != NULL )
        delete transform;
    delete exporter;
    return result;
}

void KBibTeXListView::deleteSelected()
{
    QListViewItemIterator it( this, QListViewItemIterator::Selected );
    QListViewItem *above = it.current() ->itemAbove();
    while ( it.current() )
    {
        KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );
        m_bibtexfile->deleteElement( kblvi->element() );
        it++;
        takeItem( kblvi );
    }

    if ( above )
        ensureItemVisible( above );
}

void KBibTeXListView::copySelected()
{
    kapp->clipboard() ->setText( selectedAsString() );
}

void KBibTeXListView::cutSelected()
{
    copySelected();
    deleteSelected();
}

void KBibTeXListView::paste()
{
    appendText( kapp->clipboard() ->text() );
}

bool KBibTeXListView::newElement( const QString& elementType )
{
    if ( elementType.lower() == "string" )
    {
        QString name = QString( i18n( "NewString_%1" ) ).arg( m_newElementCounter++ );
        BibTeX::BibTeXString *string = new BibTeX::BibTeXString( name );
        if ( KBibTeXStringWidget::execute( string, TRUE ) == QDialog::Accepted )
        {
            new KBibTeXListViewItem( string, this );
            m_bibtexfile->appendElement( string );
            return TRUE;
        }
        else
        {
            delete string;
            return FALSE;
        }
    }
    else if ( elementType.lower() == "comment" )
    {
        BibTeX::BibTeXComment * comment = new BibTeX::BibTeXComment( i18n( "Put your comment here..." ) );
        if ( KBibTeXCommentWidget::execute( comment, TRUE ) == QDialog::Accepted )
        {
            new KBibTeXListViewItem( comment, this );
            m_bibtexfile->appendElement( comment );
            return TRUE;
        }
        else
        {
            delete comment;
            return FALSE;
        }
    }
    else
    {
        QString name = QString( i18n( "NewEntry_%1" ) ).arg( m_newElementCounter++ );
        BibTeX::BibTeXEntry *entry = new BibTeX::BibTeXEntry( elementType, name );
        if ( KBibTeXEntryWidget::execute( entry, m_bibtexfile, TRUE ) == QDialog::Accepted )
        {
            new KBibTeXListViewItem( entry, this );
            m_bibtexfile->appendElement( entry );
            return TRUE;
        }
        else
        {
            delete entry;
            return FALSE;
        }
    }
}

bool KBibTeXListView::editElement( KBibTeXListViewItem * item, bool isReadWrite )
{
    QDialog::DialogCode dialogResult = QDialog::Rejected;

    BibTeX::BibTeXEntry * entry = dynamic_cast<BibTeX::BibTeXEntry*>( item->element() );
    if ( entry )
        dialogResult = KBibTeXEntryWidget::execute( entry, m_bibtexfile, isReadWrite );
    else
    {
        BibTeX::BibTeXComment * comment = dynamic_cast<BibTeX::BibTeXComment*>( item->element() );
        if ( comment )
            dialogResult = KBibTeXCommentWidget::execute( comment, isReadWrite );
        else
        {
            BibTeX::BibTeXString* string = dynamic_cast<BibTeX::BibTeXString*>( item->element() );
            if ( string )
                dialogResult = KBibTeXStringWidget::execute( string, isReadWrite );
        }
    }

    if ( dialogResult == QDialog::Accepted )
        item->updateItem();

    return dialogResult == QDialog::Accepted;
}

void KBibTeXListView::enabledActions()
{
    connect( header(), SIGNAL( sizeChange( int, int, int ) ), this, SLOT( columnSizeChange() ) );
    connect( header(), SIGNAL( clicked( int ) ), this, SLOT( setSortingColumn( int ) ) );
}

void KBibTeXListView::filterText( const QString & text )
{
    m_filter = text;
    updateVisiblity();
}

void KBibTeXListView::updateView()
{
    updateVisiblity();
}

void KBibTeXListView::setSortingColumn( int column )
{
    setSortColumn( column );
}

void KBibTeXListView::columnSizeChange()
{
    for ( int i = 0; i < 5; i++ )
        kbibtexsettings.mainListColumnWidth[ i ] = columnWidth( i );
}

void KBibTeXListView::dragEnterEvent( QDragEnterEvent * event )
{
    bool canAccept = QTextDrag::canDecode( event ) || QUriDrag::canDecode( event );
    qDebug( "dragEnterEvent: %s", canAccept ? "acc" : "rej" );
    event->accept( canAccept );
}

void KBibTeXListView::dropEvent( QDropEvent * event )
{
    QString text;
    QStrList urls;

    if ( QUriDrag::decode( event, urls ) && urls.count() > 0 )
    {
        KURL url( urls.first() );
        if ( url.isValid() )
            m_kbibtexPart->openURL( url );
    }
    else if ( QTextDrag::decode( event, text ) )
        appendText( text );
}

void KBibTeXListView::contentsDragEnterEvent( QDragEnterEvent * event )
{
    KListView::contentsDragEnterEvent( event );
    qDebug( "contentsDragEnterEvent" );
    event->accept( FALSE );
}

void KBibTeXListView::contentsDropEvent( QDropEvent * event )
{
    KListView::contentsDropEvent( event );
    //     dropEvent( event );
    qDebug( "ContentsDropEvent" );
    event->accept( FALSE );
}

void KBibTeXListView::startDrag()
{
    QDragObject * d = new QTextDrag( selectedAsString(), this );
    d->dragCopy();
    // do NOT delete d.
}

void KBibTeXListView::clearAll()
{
    QListViewItemIterator it( this );
    while ( it.current() )
    {
        delete it.current();
        it++;
    }
    clear();
}

void KBibTeXListView::appendText( const QString &text )
{
    QBuffer buffer;
    buffer.open( IO_WriteOnly );
    QTextStream stream( &buffer );
    stream << text;
    buffer.close();
    buffer.open( IO_ReadOnly );
    open( &buffer, TRUE );
    buffer.close();
}

void KBibTeXListView::appendBibTeXFile( BibTeX::BibTeXFile * bibtexfile )
{
    m_bibtexfile->append( bibtexfile );
}

void KBibTeXListView::showProgress( int current, int total )
{
    if ( m_progressDialog != NULL )
    {
        m_progressDialog->setTotalSteps( total );
        m_progressDialog->setProgress( current );
    }

    kapp->processEvents();
}

void KBibTeXListView::startProgress( const QString & label, QObject * progressFrom )
{
    m_progressDialog = new QProgressDialog( this );
    m_progressDialog->setLabelText( label );
    connect( progressFrom, SIGNAL( progress( int, int ) ), this, SLOT( showProgress( int, int ) ) );
    connect( m_progressDialog, SIGNAL( canceled() ), progressFrom, SLOT( cancel( ) ) );
    QApplication::setOverrideCursor( Qt::waitCursor );
}

void KBibTeXListView::endProgress( QObject * progressFrom )
{
    disconnect( progressFrom, SIGNAL( progress( int, int ) ), this, SLOT( showProgress( int, int ) ) );
    disconnect( m_progressDialog, SIGNAL( canceled() ), progressFrom, SLOT( cancel( ) ) );
    delete m_progressDialog;
    m_progressDialog = NULL;
    QApplication::restoreOverrideCursor();
}

QString KBibTeXListView::selectedAsString()
{
    QBuffer buffer;
    buffer.open( IO_WriteOnly );
    BibTeX::BibTeXFileExporter * exporter = new BibTeX::BibTeXFileExporterBibTeX();
    QListViewItemIterator it( this, QListViewItemIterator::Selected );
    while ( it.current() )
    {
        KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );
        exporter->save( &buffer, kblvi->element() );
        it++;
    }
    delete exporter;
    buffer.close();

    buffer.open( IO_ReadOnly );
    QString result = QTextStream( &buffer ).read();
    buffer.close();

    return result;
}

BibTeX::BibTeXFile *KBibTeXListView::selected()
{
    BibTeX::BibTeXFile * result = new BibTeX::BibTeXFile();

    QListViewItemIterator it( this, QListViewItemIterator::Selected );
    while ( it.current() )
    {
        KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );
        BibTeX::BibTeXElement *element = kblvi->element();

        BibTeX::BibTeXEntry *entry = dynamic_cast<BibTeX::BibTeXEntry*>( element );
        if ( entry )
            result->appendElement( new BibTeX::BibTeXEntry( entry ) );
        else
        {
            BibTeX::BibTeXString *string = dynamic_cast<BibTeX::BibTeXString*>( element );
            if ( string )
                result->appendElement( new BibTeX::BibTeXString( string ) );
            else
            {
                BibTeX::BibTeXComment *comment = dynamic_cast<BibTeX::BibTeXComment*>( element );
                if ( comment )
                    result->appendElement( new BibTeX::BibTeXComment( comment ) );
            }
        }

        it++;
    }

    return result;
}

bool KBibTeXListView::acceptDrag( QDropEvent * event ) const
{
    return QTextDrag::canDecode( event ) || QUriDrag::canDecode( event );
}

#include "kbibtexlistview.moc"
