/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qstring.h>
#include <qprocess.h>

#include <kconfig.h>
#include <kstandarddirs.h>

#include "settings.h"

Settings::Settings()
{
    searchURLs.setAutoDelete( TRUE );
    checkExternalToolsAvailable();

    m_standardDirs = new KStandardDirs();
    xsltStylesheetHTML = m_standardDirs->findResource( "data", "kbibtexpart/xslt/html.xsl" );
    if ( xsltStylesheetHTML == NULL )
        qDebug( "Could not determine filename for XSLT file" );
}

Settings::~Settings()
{
    delete m_standardDirs;
}

void Settings::load( KConfig * config )
{
    // default values
    mainListColumnWidth[ 0 ] = 64;
    mainListColumnWidth[ 1 ] = 128;
    mainListColumnWidth[ 2 ] = 320;
    mainListColumnWidth[ 3 ] = 256;
    mainListColumnWidth[ 4 ] = 64;

    config->setGroup( "FileIO" );
    encoding = ( Encoding ) config->readNumEntry( "Encoding", ( int ) encLaTeX );
    exportlanguage = config->readEntry( "ExportLanguage", "english" );
    exportbibliographystyle = config->readEntry( "ExportBibliographyStyle", "plain" );
    usedExporterHTML = ( Exporter ) config->readNumEntry( "ExporterHTML", ( int ) exporterNone );
    usedExporterEndNote = ( Exporter ) config->readNumEntry( "ExporterEndNote", ( int ) exporterNone );
    usedExporterProCite = ( Exporter ) config->readNumEntry( "ExporterProCite", ( int ) exporterNone );

    config->setGroup( "Editing" );
    enableallfields = config->readBoolEntry( "EnableAllFields", false );
    mainListSortingColumn = config->readNumEntry( "MainListSortingColumn", -1 );
    for ( int i = 0; i < 5; i++ )
        mainListColumnWidth[ i ] = config->readNumEntry( QString( "MainListColumnWidth%1" ).arg( i ), mainListColumnWidth[ i ] );
    filterHistory = config->readListEntry( "FilterHistory" );
    showComments = config->readBoolEntry( "ShowComments", TRUE );
    showStrings = config->readBoolEntry( "ShowStrings", TRUE );

    config->setGroup( "SearchURLs" );
    searchURLs.clear();
    for ( int i = 1; i < 1024; i++ )
    {
        QString descr = config->readEntry( QString( "SearchURLDescription%1" ).arg( i ), QString::null );
        QString url = config->readEntry( QString( "SearchURL%1" ).arg( i ), QString::null );
        if ( descr != QString::null && url != QString::null )
        {
            SearchURL * searchURL = new SearchURL;
            searchURL->description = descr;
            searchURL->url = url;
            searchURLs.append( searchURL );
        }
        else
            break;
    }

    if ( searchURLs.count() == 0 )
        setDefaultSearchURLs();
}

void Settings::save( KConfig * config )
{
    config->setGroup( "FileIO" );
    config->writeEntry( "Encoding", ( int ) encoding );
    config->writeEntry( "ExportLanguage", exportlanguage );
    config->writeEntry( "ExportBibliographyStyle", exportbibliographystyle );
    config->writeEntry( "ExporterHTML", ( int ) usedExporterHTML );
    config->writeEntry( "ExporterEndNote", ( int ) usedExporterEndNote );
    config->writeEntry( "ExporterProCite", ( int ) usedExporterProCite );

    config->setGroup( "Editing" );
    config->writeEntry( "EnableAllFields", enableallfields );
    config->writeEntry( "MainListSortingColumn", mainListSortingColumn );
    for ( int i = 0; i < 5; i++ )
        config->writeEntry( QString( "MainListColumnWidth%1" ).arg( i ), mainListColumnWidth[ i ] );
    config->writeEntry( "FilterHistory", filterHistory );
    config->writeEntry( "ShowComments", showComments );
    config->writeEntry( "ShowStrings", showStrings );

    config->setGroup( "SearchURLs" );
    int i = 1;
    for ( SearchURL * searchURL = searchURLs.first(); searchURL;searchURL = searchURLs.next(), i++ )
    {
        config->writeEntry( QString( "SearchURLDescription%1" ).arg( i ), searchURL->description );
        config->writeEntry( QString( "SearchURL%1" ).arg( i ), searchURL->url );
    }
}

void Settings::checkExternalToolsAvailable()
{
    bibconvAvailable = checkExternalToolAvailable( "bibconv" );
    bibtex2htmlAvailable = checkExternalToolAvailable( "bibtex2html" );
    bib2xhtmlAvailable = checkExternalToolAvailable( "bib2xhtml" );
}

bool Settings::checkExternalToolAvailable( const QString &binary )
{
    QProcess process( binary );
    if ( !process.start() )
        return FALSE;

    if ( process.normalExit() )
        return TRUE;

    if ( process.isRunning() )
    {
        process.kill();
        return TRUE;
    }

    return FALSE;
}

void Settings::setDefaultSearchURLs()
{
    searchURLs.clear();

    SearchURL *searchURL = new SearchURL;
    searchURL->description = "Google";
    searchURL->url = "http://www.google.com/search?q=%1&ie=UTF-8&oe=UTF-8";
    searchURLs.append( searchURL );

    searchURL = new SearchURL;
    searchURL->description = "Google Scholar";
    searchURL->url = "http://scholar.google.com/scholar?q=%1&ie=UTF-8&oe=UTF-8";
    searchURLs.append( searchURL );

    searchURL = new SearchURL;
    searchURL->description = "CiteSeer";
    searchURL->url = "http://citeseer.ist.psu.edu/cs?q=%1&submit=Search+Documents&cs=1";
    searchURLs.append( searchURL );

    searchURL = new SearchURL;
    searchURL->description = "PubMed";
    searchURL->url = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=pmc&cmd=search&pmfilter_Fulltext=on&pmfilter_Relevance=on&term=%1&search=Find+Articles";
    searchURLs.append( searchURL );
}
